/* String class for C, v1.00					*/
/* Copyright (c) 1999 Mark RISON, mrison@hotmail.com		*/
/* Freely distributable and modifyable for non-commercial use	*/
/* as long as this comment header is preserved			*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dstring_private.h"

#define NUL ((char) 0x00)
#define STRING_GROWSIZE 8 /* Must be positive */

#ifndef Malloc
#define Malloc(s,n) malloc(n)
#endif
#ifndef Realloc
#define Realloc(s,p,n) realloc(p,n)
#endif
#ifndef Free
#define Free(s,p) free(p)
#endif

String *string_create (const char *const s)
{
   String *string;
   string = (String *) Malloc ("string_create:String", sizeof (String));
   if (string != NULL) {

      string->string = NULL;
      string->curpos = 0;
      string->bufsize = 0;
      string->bufgrowsize = STRING_GROWSIZE;

      if (s != NULL) {

         string->string = (char *) Malloc ("string_create:char[]", strlen (s) + 1);

         if (string->string != NULL) {

            memcpy (string->string, s, strlen (s) + 1);
            string->curpos = strlen (s);
            string->bufsize = strlen (s) + 1;

         } else {

            Free ("string_create:String", string);
            string = NULL;

         }

      } else {

         string->string = (char *) Malloc ("string_create:char[]", string->bufgrowsize);

         if (string->string != NULL) {

            string->curpos = 0;
            string->bufsize = string->bufgrowsize;

         } else {

            Free ("string_create:String", string);
            string = NULL;

         }

      }

   }

   return string;
}

int string_add (String *const string, const char c)
{
   if (string->curpos == string->bufsize - 1) {

      char *newstring;
      int newbufsize;

      newbufsize = string->bufsize + string->bufgrowsize;
#ifdef DEBUG_DSTRING
      fprintf (stderr, "Extending string to size %i\n", newbufsize);
#endif
      newstring = (char *) Realloc ("string_add:grow", string->string, newbufsize);
      if (newstring == NULL)
         return 1;
      string->string = newstring;
      string->bufsize = newbufsize;

   }

   string->string[(string->curpos)++] = c;
   return 0;

}

const char *string_read (const String *const string)
{
   if (string->string != NULL)
      string->string[string->curpos] = NUL;
   return string->string;
}

void string_delete (String *const string)
{
   if (string->curpos > 0)
      string->curpos -= 1;
}

void string_clear (String *const string)
{
   string->curpos = 0;
}

int string_len (const String *const string)
{
   return string->curpos;
}

void string_destroy (String *const string)
{
   if (string != NULL) {

      Free ("string_destroy:char[]", string->string);
      Free ("string_destroy:String", string);

   }
}
